diff --git a/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/CellDesignerElementCollection.java b/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/CellDesignerElementCollection.java
index 3bf25fe732e6196cdb8fd0c8dfe47d8df50d3e16..477a446ee6c97c1994c57505e4f0eecc82caa0ac 100644
--- a/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/CellDesignerElementCollection.java
+++ b/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/CellDesignerElementCollection.java
@@ -17,6 +17,7 @@ import lcsb.mapviewer.model.map.species.Gene;
 import lcsb.mapviewer.model.map.species.Protein;
 import lcsb.mapviewer.model.map.species.Rna;
 import lcsb.mapviewer.model.map.species.Species;
+import lcsb.mapviewer.model.map.species.field.AbstractSiteModification;
 import lcsb.mapviewer.model.map.species.field.ModificationResidue;
 
 /**
@@ -98,31 +99,29 @@ public class CellDesignerElementCollection {
     }
 
     String modifications = "";
+    List<ModificationResidue> regions = new ArrayList<>();
     if (modelElement instanceof AntisenseRna) {
       AntisenseRna asAntisenseRna = ((AntisenseRna) modelElement);
-      for (ModificationResidue region : asAntisenseRna.getRegions()) {
-        modifications += region.getState();
-      }
+      regions.addAll(asAntisenseRna.getRegions());
     } else if (modelElement instanceof Gene) {
       Gene asGene = ((Gene) modelElement);
-      for (ModificationResidue region : asGene.getModificationResidues()) {
-        modifications += region.getState();
-      }
+      regions.addAll(asGene.getModificationResidues());
     } else if (modelElement instanceof Protein) {
       Protein asProtein = ((Protein) modelElement);
       modifications = asProtein.getStructuralState();
-      for (ModificationResidue region : asProtein.getModificationResidues()) {
-        modifications += region.getState();
-      }
+      regions.addAll(asProtein.getModificationResidues());
     } else if (modelElement instanceof Rna) {
       Rna asRna = ((Rna) modelElement);
-      for (ModificationResidue region : asRna.getRegions()) {
-        modifications += region.getState();
-      }
+      regions.addAll(asRna.getRegions());
     } else if (modelElement instanceof Complex) {
       Complex asComplex = ((Complex) modelElement);
       modifications = asComplex.getStructuralState();
     }
+    for (ModificationResidue region : regions) {
+      if (region instanceof AbstractSiteModification) {
+        modifications += ((AbstractSiteModification) region).getState();
+      }
+    }
 
     String complexId = "";
     String homodimer = "";
diff --git a/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerGeneRegion.java b/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerGeneRegion.java
index 732b54d2384bbc489976b1bb93dd6daae498b04f..fbd73df0314e29fb534507dfdb93aab2049999e0 100644
--- a/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerGeneRegion.java
+++ b/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerGeneRegion.java
@@ -105,9 +105,9 @@ public class CellDesignerGeneRegion implements Serializable {
     this.setIdModificationResidue(mr.getIdModificationResidue());
     this.name = mr.getName();
     this.angle = converter.getAngleForPoint(mr.getSpecies(), mr.getPosition());
-    this.state = mr.getState();
     if (mr instanceof ModificationSite) {
       this.setModificationType("MODIFICATION_SITE");
+      this.state = ((ModificationSite) mr).getState();
     } else {
       throw new InvalidArgumentException("Unknown gene modification: " + mr.getClass());
     }
@@ -342,6 +342,7 @@ public class CellDesignerGeneRegion implements Serializable {
       throw new InvalidArgumentException("Unknown gene modification type");
     } else if (modificationType.equals("MODIFICATION_SITE")) {
       result = new ModificationSite();
+      ((ModificationSite) result).setState(this.getState());
     } else {
       logger.debug(this);
       throw new InvalidArgumentException("Unknown gene modification type: " + modificationType);
@@ -354,7 +355,6 @@ public class CellDesignerGeneRegion implements Serializable {
       angle = 0.0;
     }
     result.setPosition(converter.getResidueCoordinates(element, angle));
-    result.setState(this.getState());
     return result;
   }
 
diff --git a/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerModificationResidue.java b/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerModificationResidue.java
index 3e811d1fa1b979468a87356d69fee07fc6aeeaa4..0b55ea3074fc2243fbd24ba2945be07a9f97569e 100644
--- a/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerModificationResidue.java
+++ b/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerModificationResidue.java
@@ -106,11 +106,12 @@ public class CellDesignerModificationResidue implements Serializable {
     this.idModificationResidue = mr.getIdModificationResidue();
     this.name = mr.getName();
     this.angle = converter.getAngleForPoint(mr.getSpecies(), mr.getPosition());
-    this.state = mr.getState();
     if (mr instanceof Residue) {
       this.modificationType = "RESIDUE";
+      this.state = ((Residue) mr).getState();
     } else if (mr instanceof ModificationSite) {
       this.modificationType = "MODIFICATION_SITE";
+      this.state = ((ModificationSite) mr).getState();
     } else if (mr instanceof CodingRegion) {
       this.modificationType = "CODING_REGION";
     } else if (mr instanceof ProteinBindingDomain) {
@@ -347,6 +348,7 @@ public class CellDesignerModificationResidue implements Serializable {
       throw new InvalidArgumentException("No type information for modification: " + idModificationResidue);
     } else if (modificationType.equals("RESIDUE")) {
       result = new Residue();
+      ((Residue) result).setState(this.getState());
     } else {
       throw new InvalidArgumentException("Unknown protein modification type: " + modificationType);
     }
@@ -358,7 +360,6 @@ public class CellDesignerModificationResidue implements Serializable {
       angle = 0.0;
     }
     result.setPosition(converter.getResidueCoordinates(element, angle));
-    result.setState(this.getState());
     return result;
   }
 
diff --git a/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerRnaRegion.java b/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerRnaRegion.java
index 3c287ec229ca0a6e49e901f4257735395de31ae9..ff2e2a803c25ad614ca1edf98a45b0827852aafa 100644
--- a/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerRnaRegion.java
+++ b/converter-CellDesigner/src/main/java/lcsb/mapviewer/converter/model/celldesigner/structure/fields/CellDesignerRnaRegion.java
@@ -275,6 +275,7 @@ public class CellDesignerRnaRegion implements Serializable {
       throw new InvalidArgumentException("No type information for modification: " + idRnaRegion);
     } else if (modificationType.equals("MODIFICATION_SITE")) {
       result = new ModificationSite();
+      ((ModificationSite) result).setState(this.state);
     } else if (modificationType.equals("CODING_REGION")) {
       result = new CodingRegion();
       ((CodingRegion) result).setWidth(element.getWidth() * size);
@@ -288,7 +289,6 @@ public class CellDesignerRnaRegion implements Serializable {
     result.setIdModificationResidue(this.idRnaRegion);
     result.setPosition(new Point2D.Double(
         element.getX() + element.getWidth() / 4.0 + element.getWidth() * 3.0 / 4.0 * pos, element.getY()));
-    result.setState(this.state);
     result.setName(this.name);
     return result;
   }
diff --git a/converter-CellDesigner/src/test/java/lcsb/mapviewer/converter/model/celldesigner/ComplexParserTests.java b/converter-CellDesigner/src/test/java/lcsb/mapviewer/converter/model/celldesigner/ComplexParserTests.java
index d925d5ec48aa62a669f55147e42f7371a7cbc5e6..60ffcd0ccdef733544c527042c666298bd7457a2 100644
--- a/converter-CellDesigner/src/test/java/lcsb/mapviewer/converter/model/celldesigner/ComplexParserTests.java
+++ b/converter-CellDesigner/src/test/java/lcsb/mapviewer/converter/model/celldesigner/ComplexParserTests.java
@@ -33,8 +33,10 @@ import lcsb.mapviewer.model.map.species.GenericProtein;
 import lcsb.mapviewer.model.map.species.Protein;
 import lcsb.mapviewer.model.map.species.Rna;
 import lcsb.mapviewer.model.map.species.Species;
+import lcsb.mapviewer.model.map.species.field.AbstractSiteModification;
 import lcsb.mapviewer.model.map.species.field.ModificationResidue;
 import lcsb.mapviewer.model.map.species.field.ModificationState;
+import lcsb.mapviewer.model.map.species.field.Residue;
 import lcsb.mapviewer.modelutils.map.ElementUtils;
 
 public class ComplexParserTests extends CellDesignerTestFunctions {
@@ -99,7 +101,7 @@ public class ComplexParserTests extends CellDesignerTestFunctions {
     try {
       Model model = getModelForFile("testFiles/elements_with_kinetic_data.xml");
       model.setName(null);
-      
+
       String xml = new CellDesignerXmlParser().toXml(model);
       ByteArrayInputStream bais = new ByteArrayInputStream(xml.getBytes());
       Model model2 = new CellDesignerXmlParser()
@@ -118,7 +120,7 @@ public class ComplexParserTests extends CellDesignerTestFunctions {
     try {
       Model model = getModelForFile("testFiles/reactions/kinetics.xml");
       model.setName(null);
-      
+
       String xml = new CellDesignerXmlParser().toXml(model);
       ByteArrayInputStream bais = new ByteArrayInputStream(xml.getBytes());
       Model model2 = new CellDesignerXmlParser()
@@ -222,12 +224,14 @@ public class ComplexParserTests extends CellDesignerTestFunctions {
     try {
       model = getModelForFile("testFiles/problematic/acetyled_protein.xml");
 
-      Set<ModificationState> residues = new HashSet<ModificationState>();
+      Set<ModificationState> residues = new HashSet<>();
       for (Element element : model.getElements()) {
         if (element instanceof Species) {
-          Protein p = (Protein) element;
-          for (ModificationResidue mr : p.getModificationResidues()) {
-            residues.add(mr.getState());
+          Protein protein = (Protein) element;
+          for (ModificationResidue mr : protein.getModificationResidues()) {
+            if (mr instanceof AbstractSiteModification) {
+              residues.add(((AbstractSiteModification) mr).getState());
+            }
           }
         }
       }
@@ -235,8 +239,9 @@ public class ComplexParserTests extends CellDesignerTestFunctions {
       // of residues
       assertEquals(2, residues.size());
 
-      assertEquals(ModificationState.ACETYLATED,
-          ((Protein) model.getElementByElementId("sa2")).getModificationResidues().get(0).getState());
+      AbstractSiteModification modification = (AbstractSiteModification) ((Protein) model.getElementByElementId("sa2"))
+          .getModificationResidues().get(0);
+      assertEquals(ModificationState.ACETYLATED, modification.getState());
     } catch (Exception e) {
       e.printStackTrace();
       throw e;
@@ -278,8 +283,8 @@ public class ComplexParserTests extends CellDesignerTestFunctions {
       model = getModelForFile("testFiles/problematic/problematic_acetylation.xml");
       Protein p1 = (Protein) model.getElementByElementId("sa73");
       Protein p2 = (Protein) model.getElementByElementId("sa27");
-      assertEquals(ModificationState.ACETYLATED, p1.getModificationResidues().get(0).getState());
-      assertFalse(ModificationState.ACETYLATED.equals(p2.getModificationResidues().get(0).getState()));
+      assertEquals(ModificationState.ACETYLATED, ((Residue) p1.getModificationResidues().get(0)).getState());
+      assertFalse(ModificationState.ACETYLATED.equals(((Residue) p2.getModificationResidues().get(0)).getState()));
 
     } catch (Exception e) {
       e.printStackTrace();
diff --git a/converter-CellDesigner/src/test/java/lcsb/mapviewer/converter/model/celldesigner/species/GeneXmlParserTest.java b/converter-CellDesigner/src/test/java/lcsb/mapviewer/converter/model/celldesigner/species/GeneXmlParserTest.java
index da9e20d35c2ed3bcdd19d9aa4d5413937c798d3d..346852b1316ba079d3eccc095ccd1141c84cc693 100644
--- a/converter-CellDesigner/src/test/java/lcsb/mapviewer/converter/model/celldesigner/species/GeneXmlParserTest.java
+++ b/converter-CellDesigner/src/test/java/lcsb/mapviewer/converter/model/celldesigner/species/GeneXmlParserTest.java
@@ -28,7 +28,7 @@ import lcsb.mapviewer.model.map.model.Model;
 import lcsb.mapviewer.model.map.species.Gene;
 import lcsb.mapviewer.model.map.species.GenericProtein;
 import lcsb.mapviewer.model.map.species.Protein;
-import lcsb.mapviewer.model.map.species.field.ModificationResidue;
+import lcsb.mapviewer.model.map.species.field.ModificationSite;
 import lcsb.mapviewer.model.map.species.field.ModificationState;
 import lcsb.mapviewer.model.map.species.field.Residue;
 
@@ -96,7 +96,7 @@ public class GeneXmlParserTest extends CellDesignerTestFunctions {
       Model model = getModelForFile("testFiles/problematic/phosphorylated_gene.xml");
       Gene gene = (Gene) model.getElementByElementId("sa1");
       assertEquals(1, gene.getModificationResidues().size());
-      ModificationResidue residue = gene.getModificationResidues().get(0);
+      ModificationSite residue = (ModificationSite) gene.getModificationResidues().get(0);
       assertEquals(ModificationState.PHOSPHORYLATED, residue.getState());
       assertEquals("some name", residue.getName());
 
diff --git a/converter-SBGNML/src/main/java/lcsb/mapviewer/converter/model/sbgnml/SbgnmlXmlExporter.java b/converter-SBGNML/src/main/java/lcsb/mapviewer/converter/model/sbgnml/SbgnmlXmlExporter.java
index 1939837458b86bb6b0224bba37052a0f4e5e3d8d..39b313a2589585494b053bd25fd6f23cf36cb1b6 100644
--- a/converter-SBGNML/src/main/java/lcsb/mapviewer/converter/model/sbgnml/SbgnmlXmlExporter.java
+++ b/converter-SBGNML/src/main/java/lcsb/mapviewer/converter/model/sbgnml/SbgnmlXmlExporter.java
@@ -69,6 +69,7 @@ import lcsb.mapviewer.model.map.species.SimpleMolecule;
 import lcsb.mapviewer.model.map.species.Species;
 import lcsb.mapviewer.model.map.species.TruncatedProtein;
 import lcsb.mapviewer.model.map.species.Unknown;
+import lcsb.mapviewer.model.map.species.field.AbstractSiteModification;
 import lcsb.mapviewer.model.map.species.field.ModificationResidue;
 import lcsb.mapviewer.modelutils.map.ElementUtils;
 
@@ -430,11 +431,14 @@ public class SbgnmlXmlExporter {
     glyph.setId(mr.getIdModificationResidue());
     glyph.setClazz(GlyphClazz.STATE_VARIABLE.getClazz());
 
-    if (mr.getState() != null) {
-      Glyph.State state = new Glyph.State();
-      state.setValue(mr.getState().getAbbreviation());
-      state.setVariable(mr.getName());
-      glyph.setState(state);
+    if (mr instanceof AbstractSiteModification) {
+      AbstractSiteModification modification = (AbstractSiteModification) mr;
+      if (modification.getState() != null) {
+        Glyph.State state = new Glyph.State();
+        state.setValue(modification.getState().getAbbreviation());
+        state.setVariable(mr.getName());
+        glyph.setState(state);
+      }
     }
 
     Bbox bbox = new Bbox();
diff --git a/converter-SBGNML/src/main/java/lcsb/mapviewer/converter/model/sbgnml/SbgnmlXmlParser.java b/converter-SBGNML/src/main/java/lcsb/mapviewer/converter/model/sbgnml/SbgnmlXmlParser.java
index 9bace297459326db8595c9ec7d816d9dea5478e1..c38a6e2e9482a7509ab690f8e0306f6303f6a03c 100644
--- a/converter-SBGNML/src/main/java/lcsb/mapviewer/converter/model/sbgnml/SbgnmlXmlParser.java
+++ b/converter-SBGNML/src/main/java/lcsb/mapviewer/converter/model/sbgnml/SbgnmlXmlParser.java
@@ -800,7 +800,7 @@ public class SbgnmlXmlParser {
       throw new Exception("Only macromolecule elements can have state variables.");
     }
     Protein protein = (Protein) newSpecies;
-    ModificationResidue mr = new Residue();
+    Residue mr = new Residue();
 
     mr.setSpecies(protein);
     mr.setIdModificationResidue(unitOfInformationGlyph.getId());
diff --git a/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/GeneConverter.java b/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/GeneConverter.java
index 41c9002e5c023ca37369d5c974c366c525cf7a11..15bc9862500a30feb9435b7ef34eb6fd5e644623 100644
--- a/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/GeneConverter.java
+++ b/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/GeneConverter.java
@@ -19,6 +19,7 @@ import lcsb.mapviewer.model.map.layout.ColorSchema;
 import lcsb.mapviewer.model.map.species.Gene;
 import lcsb.mapviewer.model.map.species.Species;
 import lcsb.mapviewer.model.map.species.field.ModificationResidue;
+import lcsb.mapviewer.model.map.species.field.ModificationSite;
 import lcsb.mapviewer.model.map.species.field.ModificationState;
 
 /**
@@ -72,7 +73,7 @@ public class GeneConverter extends SpeciesConverter<Gene> {
    * 
    * @param gene
    *          object that is 'parent' of the residue
-   * @param mr
+   * @param mr2
    *          modification to be drawn
    * @param graphics
    *          - where the modification should be drawn
@@ -84,39 +85,42 @@ public class GeneConverter extends SpeciesConverter<Gene> {
    */
   private void drawModification(final Gene gene, final ModificationResidue mr, final Graphics2D graphics,
       final boolean drawEmptyModification, final boolean drawDescription) {
-    if ((!drawEmptyModification) && (mr.getState() == null)) {
-      return;
-    }
-    double diameter = DEFAULT_MODIFICATION_DIAMETER;
-
-    double y = gene.getY();
-
-    Point2D p = mr.getPosition();
-    p.setLocation(p.getX(), p.getY() - DEFAULT_MODIFICATION_DIAMETER);
-
-    Ellipse2D ellipse = new Ellipse2D.Double(p.getX() - diameter / 2, p.getY() - diameter / 2, diameter, diameter);
-    Color c = graphics.getColor();
-    graphics.setColor(Color.WHITE);
-    graphics.fill(ellipse);
-    graphics.setColor(c);
-    graphics.draw(ellipse);
-    graphics.drawLine((int) p.getX(), (int) (p.getY() + diameter / 2), (int) p.getX(), (int) y);
-
-    String text = mr.getName();
-    if (!text.equals("") && drawDescription) {
-      double textWidth = graphics.getFontMetrics().stringWidth(text);
-      Point2D p2 = new Point2D.Double(p.getX() - textWidth / 2, p.getY() - DEFAULT_SPECIES_MODIFIER_FONT_SIZE);
-      graphics.drawString(text, (int) p2.getX(), (int) p2.getY());
-    }
-    ModificationState state = mr.getState();
-    if (state != null) {
-      String str = state.getAbbreviation();
-      Font tmpFont = graphics.getFont();
-      graphics.setFont(getStructuralFont());
-      double textX = p.getX() - graphics.getFontMetrics().stringWidth(str) / 2;
-      double textY = p.getY() + graphics.getFontMetrics().getAscent() / 2;
-      graphics.drawString(str, (int) textX, (int) textY);
-      graphics.setFont(tmpFont);
+    if (mr instanceof ModificationSite) {
+      ModificationSite modificationSite = (ModificationSite) mr;
+      if (!drawEmptyModification && modificationSite.getState() == null) {
+        return;
+      }
+      double diameter = DEFAULT_MODIFICATION_DIAMETER;
+
+      double y = gene.getY();
+
+      Point2D p = modificationSite.getPosition();
+      p.setLocation(p.getX(), p.getY() - DEFAULT_MODIFICATION_DIAMETER);
+
+      Ellipse2D ellipse = new Ellipse2D.Double(p.getX() - diameter / 2, p.getY() - diameter / 2, diameter, diameter);
+      Color c = graphics.getColor();
+      graphics.setColor(Color.WHITE);
+      graphics.fill(ellipse);
+      graphics.setColor(c);
+      graphics.draw(ellipse);
+      graphics.drawLine((int) p.getX(), (int) (p.getY() + diameter / 2), (int) p.getX(), (int) y);
+
+      String text = modificationSite.getName();
+      if (!text.equals("") && drawDescription) {
+        double textWidth = graphics.getFontMetrics().stringWidth(text);
+        Point2D p2 = new Point2D.Double(p.getX() - textWidth / 2, p.getY() - DEFAULT_SPECIES_MODIFIER_FONT_SIZE);
+        graphics.drawString(text, (int) p2.getX(), (int) p2.getY());
+      }
+      ModificationState state = modificationSite.getState();
+      if (state != null) {
+        String str = state.getAbbreviation();
+        Font tmpFont = graphics.getFont();
+        graphics.setFont(getStructuralFont());
+        double textX = p.getX() - graphics.getFontMetrics().stringWidth(str) / 2;
+        double textY = p.getY() + graphics.getFontMetrics().getAscent() / 2;
+        graphics.drawString(str, (int) textX, (int) textY);
+        graphics.setFont(tmpFont);
+      }
     }
 
   }
diff --git a/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/ProteinConverter.java b/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/ProteinConverter.java
index 8c1ae521f2cbb531275d666ab137613deedb0a92..5a35913a28cff5d16ee82cda56b41f54325dc820 100644
--- a/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/ProteinConverter.java
+++ b/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/ProteinConverter.java
@@ -29,7 +29,9 @@ import lcsb.mapviewer.model.map.species.ReceptorProtein;
 import lcsb.mapviewer.model.map.species.Species;
 import lcsb.mapviewer.model.map.species.TruncatedProtein;
 import lcsb.mapviewer.model.map.species.field.ModificationResidue;
+import lcsb.mapviewer.model.map.species.field.ModificationSite;
 import lcsb.mapviewer.model.map.species.field.ModificationState;
+import lcsb.mapviewer.model.map.species.field.Residue;
 import lcsb.mapviewer.modelutils.map.ElementUtils;
 
 /**
@@ -303,29 +305,31 @@ public class ProteinConverter extends SpeciesConverter<Protein> {
    */
   private void drawModification(final Protein protein, final ModificationResidue mr, final Graphics2D graphics,
       final boolean drawEmptyModification) {
-    if ((!drawEmptyModification) && (mr.getState() == null)) {
-      return;
-    }
-    double diameter = DEFAULT_MODIFICATION_DIAMETER;
-    Point2D p = mr.getPosition();
-    Ellipse2D ellipse = new Ellipse2D.Double(p.getX() - diameter / 2, p.getY() - diameter / 2, diameter, diameter);
-    Color c = graphics.getColor();
-    graphics.setColor(Color.WHITE);
-    graphics.fill(ellipse);
-    graphics.setColor(c);
-    graphics.draw(ellipse);
-
-    ModificationState state = mr.getState();
-    if (state != null) {
-      String str = state.getAbbreviation();
-      Font tmpFont = graphics.getFont();
-      graphics.setFont(getStructuralFont());
-      double x = p.getX() - graphics.getFontMetrics().stringWidth(str) / 2;
-      double y = p.getY() + graphics.getFontMetrics().getAscent() / 2;
-      graphics.drawString(str, (int) x, (int) y);
-      graphics.setFont(tmpFont);
+    if (mr instanceof Residue) {
+      Residue residue = (Residue) mr;
+      if (!drawEmptyModification && residue.getState() == null) {
+        return;
+      }
+      double diameter = DEFAULT_MODIFICATION_DIAMETER;
+      Point2D p = residue.getPosition();
+      Ellipse2D ellipse = new Ellipse2D.Double(p.getX() - diameter / 2, p.getY() - diameter / 2, diameter, diameter);
+      Color c = graphics.getColor();
+      graphics.setColor(Color.WHITE);
+      graphics.fill(ellipse);
+      graphics.setColor(c);
+      graphics.draw(ellipse);
+
+      ModificationState state = residue.getState();
+      if (state != null) {
+        String str = state.getAbbreviation();
+        Font tmpFont = graphics.getFont();
+        graphics.setFont(getStructuralFont());
+        double x = p.getX() - graphics.getFontMetrics().stringWidth(str) / 2;
+        double y = p.getY() + graphics.getFontMetrics().getAscent() / 2;
+        graphics.drawString(str, (int) x, (int) y);
+        graphics.setFont(tmpFont);
+      }
     }
-
   }
 
   /**
diff --git a/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/RnaConverter.java b/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/RnaConverter.java
index f0fbd05c56a1e00d0b8b14ce9b679b2df94acad5..16533c7d5b7173e85e0261fdfa9eb65f8f99f86f 100644
--- a/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/RnaConverter.java
+++ b/converter-graphics/src/main/java/lcsb/mapviewer/converter/graphics/bioEntity/element/species/RnaConverter.java
@@ -26,6 +26,7 @@ import lcsb.mapviewer.model.map.layout.ColorSchema;
 import lcsb.mapviewer.model.map.species.Rna;
 import lcsb.mapviewer.model.map.species.Species;
 import lcsb.mapviewer.model.map.species.field.ModificationResidue;
+import lcsb.mapviewer.model.map.species.field.ModificationSite;
 import lcsb.mapviewer.model.map.species.field.ModificationState;
 
 /**
@@ -113,42 +114,44 @@ public class RnaConverter extends SpeciesConverter<Rna> {
    *          flag that indicates if we should draw description of the
    *          modification
    */
-  private void drawModification(final Rna rna, final ModificationResidue region, final Graphics2D graphics,
+  private void drawModification(final Rna rna, final ModificationResidue mr, final Graphics2D graphics,
       final boolean drawEmptyModification, final boolean drawDescription) {
-    if ((!drawEmptyModification) && (region.getState() == null)) {
-      return;
+    if (mr instanceof ModificationSite) {
+      ModificationSite modificationSite = (ModificationSite) mr;
+      if (!drawEmptyModification && modificationSite.getState() == null) {
+        return;
+      }
+      double diameter = DEFAULT_MODIFICATION_DIAMETER;
+
+      double y = rna.getY();
+
+      Point2D p = new Point2D.Double(modificationSite.getPosition().getX(), modificationSite.getPosition().getY() - diameter);
+
+      Ellipse2D ellipse = new Ellipse2D.Double(p.getX() - diameter / 2, p.getY() - diameter / 2, diameter, diameter);
+      Color c = graphics.getColor();
+      graphics.setColor(Color.WHITE);
+      graphics.fill(ellipse);
+      graphics.setColor(c);
+      graphics.draw(ellipse);
+      graphics.drawLine((int) p.getX(), (int) (p.getY() + diameter / 2), (int) p.getX(), (int) y);
+
+      String text = modificationSite.getName();
+      if (!text.equals("") && drawDescription) {
+        double textWidth = graphics.getFontMetrics().stringWidth(text);
+        Point2D p2 = new Point2D.Double(p.getX() - textWidth / 2, p.getY() - DEFAULT_SPECIES_MODIFIER_FONT_SIZE);
+        graphics.drawString(text, (int) p2.getX(), (int) p2.getY());
+      }
+      ModificationState state = modificationSite.getState();
+      if (state != null) {
+        String str = state.getAbbreviation();
+        Font tmpFont = graphics.getFont();
+        graphics.setFont(getStructuralFont());
+        double textX = p.getX() - graphics.getFontMetrics().stringWidth(str) / 2;
+        double textY = p.getY() + graphics.getFontMetrics().getAscent() / 2;
+        graphics.drawString(str, (int) textX, (int) textY);
+        graphics.setFont(tmpFont);
+      }
     }
-    double diameter = DEFAULT_MODIFICATION_DIAMETER;
-
-    double y = rna.getY();
-
-    Point2D p = new Point2D.Double(region.getPosition().getX(), region.getPosition().getY() - diameter);
-
-    Ellipse2D ellipse = new Ellipse2D.Double(p.getX() - diameter / 2, p.getY() - diameter / 2, diameter, diameter);
-    Color c = graphics.getColor();
-    graphics.setColor(Color.WHITE);
-    graphics.fill(ellipse);
-    graphics.setColor(c);
-    graphics.draw(ellipse);
-    graphics.drawLine((int) p.getX(), (int) (p.getY() + diameter / 2), (int) p.getX(), (int) y);
-
-    String text = region.getName();
-    if (!text.equals("") && drawDescription) {
-      double textWidth = graphics.getFontMetrics().stringWidth(text);
-      Point2D p2 = new Point2D.Double(p.getX() - textWidth / 2, p.getY() - DEFAULT_SPECIES_MODIFIER_FONT_SIZE);
-      graphics.drawString(text, (int) p2.getX(), (int) p2.getY());
-    }
-    ModificationState state = region.getState();
-    if (state != null) {
-      String str = state.getAbbreviation();
-      Font tmpFont = graphics.getFont();
-      graphics.setFont(getStructuralFont());
-      double textX = p.getX() - graphics.getFontMetrics().stringWidth(str) / 2;
-      double textY = p.getY() + graphics.getFontMetrics().getAscent() / 2;
-      graphics.drawString(str, (int) textX, (int) textY);
-      graphics.setFont(tmpFont);
-    }
-
   }
 
 }
diff --git a/model/src/main/java/lcsb/mapviewer/model/map/species/field/AbstractRegionModification.java b/model/src/main/java/lcsb/mapviewer/model/map/species/field/AbstractRegionModification.java
new file mode 100644
index 0000000000000000000000000000000000000000..328407e43e4e6514aee5cb28091847b455a55195
--- /dev/null
+++ b/model/src/main/java/lcsb/mapviewer/model/map/species/field/AbstractRegionModification.java
@@ -0,0 +1,83 @@
+package lcsb.mapviewer.model.map.species.field;
+
+import java.text.DecimalFormat;
+
+import javax.persistence.Column;
+import javax.persistence.DiscriminatorValue;
+import javax.persistence.Entity;
+
+import lcsb.mapviewer.common.exception.InvalidArgumentException;
+
+@Entity
+@DiscriminatorValue("ABSTRACT_REGION_MODIFICATION")
+public abstract class AbstractRegionModification extends ModificationResidue {
+
+  /**
+   * 
+   */
+  private static final long serialVersionUID = 1L;
+
+  /**
+   * Default size of the object (in graphical representation).
+   */
+  private static final double DEFAULT_SIZE = 10;
+
+  /**
+   * Width of the region in the graphic representation.
+   */
+  @Column(name = "width")
+  private double width = DEFAULT_SIZE;
+
+  public AbstractRegionModification() {
+    super();
+  }
+
+  public AbstractRegionModification(AbstractRegionModification mr) {
+    super(mr);
+    this.width = mr.getWidth();
+  }
+
+  @Override
+  public String toString() {
+    DecimalFormat format = new DecimalFormat("#.##");
+    String position;
+    if (getPosition() == null) {
+      position = null;
+    } else {
+      position = "Point2D[" + format.format(getPosition().getX()) + "," + format.format(getPosition().getY()) + "]";
+    }
+    String result = getIdModificationResidue() + "," + getName() + "," + getWidth() + "," + position;
+    return result;
+  }
+
+  public double getWidth() {
+    return width;
+  }
+
+  public void setWidth(double width) {
+    this.width = width;
+  }
+
+  /**
+   * Update data in this object from parameter (only if values in parameter object
+   * are valid).
+   * 
+   * @param mr
+   *          object from which we are updating data
+   */
+  public void update(AbstractRegionModification mr) {
+    if (this.getIdModificationResidue() != null && !this.getIdModificationResidue().equals("")
+        && !this.getIdModificationResidue().equals(mr.getIdModificationResidue())) {
+      throw new InvalidArgumentException("Cannot update from mr with different id");
+    }
+    if (mr.getWidth() > 0) {
+      this.setWidth(mr.getWidth());
+    }
+    if (mr.getName() != null) {
+      this.setName(mr.getName());
+    }
+    if (mr.getPosition() != null) {
+      this.setPosition(mr.getPosition());
+    }
+  }
+}
\ No newline at end of file
diff --git a/model/src/main/java/lcsb/mapviewer/model/map/species/field/AbstractSiteModification.java b/model/src/main/java/lcsb/mapviewer/model/map/species/field/AbstractSiteModification.java
new file mode 100644
index 0000000000000000000000000000000000000000..d0249cea6281efb73da01a857a48fe80c72e932d
--- /dev/null
+++ b/model/src/main/java/lcsb/mapviewer/model/map/species/field/AbstractSiteModification.java
@@ -0,0 +1,51 @@
+package lcsb.mapviewer.model.map.species.field;
+
+import java.text.DecimalFormat;
+
+import javax.persistence.DiscriminatorValue;
+import javax.persistence.Entity;
+import javax.persistence.EnumType;
+import javax.persistence.Enumerated;
+
+@Entity
+@DiscriminatorValue("ABSTRACT_SITE_MODIFICATION")
+public abstract class AbstractSiteModification extends ModificationResidue {
+
+  /**
+   * 
+   */
+  private static final long serialVersionUID = 1L;
+  
+  /**
+   * State in which this modification is.
+   */
+  @Enumerated(EnumType.STRING)
+  protected ModificationState state = null;
+
+  public AbstractSiteModification() {
+    super();
+  }
+
+  public AbstractSiteModification(AbstractSiteModification mr) {
+    super(mr);
+    this.state = mr.getState();
+  }
+
+  public ModificationState getState() {
+    return state;
+  }
+
+  public void setState(ModificationState state) {
+    this.state = state;
+  }
+
+  @Override
+  public String toString() {
+    DecimalFormat format = new DecimalFormat("#.##");
+    String result = getIdModificationResidue() + "," + getName() + "," + getState() + ",Point2D["
+        + format.format(getPosition().getX()) + "," + format.format(getPosition().getY()) + "]";
+    return result;
+  }
+
+
+}
\ No newline at end of file
diff --git a/model/src/main/java/lcsb/mapviewer/model/map/species/field/ModificationResidue.java b/model/src/main/java/lcsb/mapviewer/model/map/species/field/ModificationResidue.java
index 1049408c123eb6609b38ffa6e5f44f023cbd4a55..4c8e7b3e37488f08699432ae4285de7fe39e745f 100644
--- a/model/src/main/java/lcsb/mapviewer/model/map/species/field/ModificationResidue.java
+++ b/model/src/main/java/lcsb/mapviewer/model/map/species/field/ModificationResidue.java
@@ -2,7 +2,6 @@ package lcsb.mapviewer.model.map.species.field;
 
 import java.awt.geom.Point2D;
 import java.io.Serializable;
-import java.text.DecimalFormat;
 
 import javax.persistence.CascadeType;
 import javax.persistence.Column;
@@ -10,8 +9,6 @@ import javax.persistence.DiscriminatorColumn;
 import javax.persistence.DiscriminatorType;
 import javax.persistence.DiscriminatorValue;
 import javax.persistence.Entity;
-import javax.persistence.EnumType;
-import javax.persistence.Enumerated;
 import javax.persistence.FetchType;
 import javax.persistence.GeneratedValue;
 import javax.persistence.GenerationType;
@@ -29,9 +26,7 @@ import lcsb.mapviewer.common.exception.NotImplementedException;
 import lcsb.mapviewer.model.map.species.Species;
 
 /**
- * This class represent modification residue in protein and gene. However, it is
- * sometimes also used for storing information about AntisenseRna/Rna regions...
- * (due to CellDesigner xml strange structure).
+ * This class represent modification residue in a {@link Species}.
  * 
  * @author Piotr Gawron
  * 
@@ -74,12 +69,6 @@ public class ModificationResidue implements Serializable {
    */
   private String name = "";
 
-  /**
-   * State in which this modification is.
-   */
-  @Enumerated(EnumType.STRING)
-  private ModificationState state = null;
-
   @Column(name = "position")
   @Type(type = "lcsb.mapviewer.persist.mapper.Point2DMapper")
   private Point2D position = null;
@@ -106,18 +95,9 @@ public class ModificationResidue implements Serializable {
   public ModificationResidue(ModificationResidue mr) {
     this.idModificationResidue = mr.idModificationResidue;
     this.name = mr.name;
-    this.state = mr.state;
     this.position = mr.position;
   }
 
-  @Override
-  public String toString() {
-    DecimalFormat format = new DecimalFormat("#.##");
-    String result = getIdModificationResidue() + "," + getName() + "," + getState() + ",Point2D["
-        + format.format(getPosition().getX()) + "," + format.format(getPosition().getY()) + "]";
-    return result;
-  }
-
   /**
    * @return the idModificationResidue
    * @see #id
@@ -169,23 +149,6 @@ public class ModificationResidue implements Serializable {
     this.name = name;
   }
 
-  /**
-   * @return the state
-   * @see #state
-   */
-  public ModificationState getState() {
-    return state;
-  }
-
-  /**
-   * @param state
-   *          the state to set
-   * @see #state
-   */
-  public void setState(ModificationState state) {
-    this.state = state;
-  }
-
   /**
    * @return the species
    * @see #species
diff --git a/model/src/main/java/lcsb/mapviewer/model/map/species/field/ModificationSite.java b/model/src/main/java/lcsb/mapviewer/model/map/species/field/ModificationSite.java
index 390c96377949e573040610a8a5aaf2ef1c1f0aaa..52b13b58659872ee34278be3e53a197531cf5fa7 100644
--- a/model/src/main/java/lcsb/mapviewer/model/map/species/field/ModificationSite.java
+++ b/model/src/main/java/lcsb/mapviewer/model/map/species/field/ModificationSite.java
@@ -4,10 +4,26 @@ import javax.persistence.DiscriminatorValue;
 import javax.persistence.Entity;
 
 import lcsb.mapviewer.common.exception.NotImplementedException;
+import lcsb.mapviewer.model.map.species.AntisenseRna;
+import lcsb.mapviewer.model.map.species.Gene;
+import lcsb.mapviewer.model.map.species.Rna;
+import lcsb.mapviewer.model.map.species.Species;
 
+/**
+ * This structure contains information about Modification Site for one of the
+ * following {@link Species}:
+ * <ul>
+ * <li>{@link Rna}</li>
+ * <li>{@link AntisenseRna}</li>
+ * <li>{@link Gene}</li>
+ * </ul>
+ * 
+ * @author Piotr Gawron
+ * 
+ */
 @Entity
 @DiscriminatorValue("MODIFICATION_SITE")
-public class ModificationSite extends ModificationResidue {
+public class ModificationSite extends AbstractSiteModification {
 
   /**
    * 
@@ -17,11 +33,10 @@ public class ModificationSite extends ModificationResidue {
   public ModificationSite() {
     super();
   }
-  
-  public ModificationSite(ModificationSite residue) {
-    super(residue);
+
+  public ModificationSite(ModificationSite site) {
+    super(site);
   }
-  
 
   /**
    * Creates copy of the object.
diff --git a/model/src/main/java/lcsb/mapviewer/model/map/species/field/Residue.java b/model/src/main/java/lcsb/mapviewer/model/map/species/field/Residue.java
index b03b7510041a6b57182ec2e42bca59b17d4c84cc..d93cc8956d07f82c5b61648d481c9d6de4c20b25 100644
--- a/model/src/main/java/lcsb/mapviewer/model/map/species/field/Residue.java
+++ b/model/src/main/java/lcsb/mapviewer/model/map/species/field/Residue.java
@@ -4,10 +4,17 @@ import javax.persistence.DiscriminatorValue;
 import javax.persistence.Entity;
 
 import lcsb.mapviewer.common.exception.NotImplementedException;
+import lcsb.mapviewer.model.map.species.Protein;
 
+/**
+ * This structure contains information about Residue for one of {@link Protein}.
+ * 
+ * @author Piotr Gawron
+ * 
+ */
 @Entity
 @DiscriminatorValue("RESIDUE")
-public class Residue extends ModificationResidue {
+public class Residue extends AbstractSiteModification {
 
   /**
    * 
@@ -17,11 +24,10 @@ public class Residue extends ModificationResidue {
   public Residue() {
     super();
   }
-  
+
   public Residue(Residue residue) {
     super(residue);
   }
-  
 
   /**
    * Creates copy of the object.
diff --git a/model/src/test/java/lcsb/mapviewer/model/map/species/GeneComparatorTest.java b/model/src/test/java/lcsb/mapviewer/model/map/species/GeneComparatorTest.java
index 34dc67ef016c6cf8baacd2708329f6b7dff3a7a8..95104c0be61d61a9a7db9307787413c0661fabc1 100644
--- a/model/src/test/java/lcsb/mapviewer/model/map/species/GeneComparatorTest.java
+++ b/model/src/test/java/lcsb/mapviewer/model/map/species/GeneComparatorTest.java
@@ -12,6 +12,7 @@ import org.junit.Test;
 import org.mockito.Mockito;
 
 import lcsb.mapviewer.common.exception.NotImplementedException;
+import lcsb.mapviewer.model.map.species.field.AbstractSiteModification;
 import lcsb.mapviewer.model.map.species.field.ModificationResidue;
 import lcsb.mapviewer.model.map.species.field.ModificationSite;
 import lcsb.mapviewer.model.map.species.field.ModificationState;
@@ -82,7 +83,7 @@ public class GeneComparatorTest {
     Gene result = new Gene();
     result.setHypothetical(true);
 
-    ModificationSite residue = new ModificationSite();
+    AbstractSiteModification residue = new ModificationSite();
     result.addModificationResidue(residue);
 
     residue.setIdModificationResidue("a");
diff --git a/model/src/test/java/lcsb/mapviewer/model/map/species/ProteinComparatorTest.java b/model/src/test/java/lcsb/mapviewer/model/map/species/ProteinComparatorTest.java
index 4f3a722d8d01eaa390b20da25738a533ec00a556..e07e6d0e81ea684b57e326e9463dc66c13aed0b5 100644
--- a/model/src/test/java/lcsb/mapviewer/model/map/species/ProteinComparatorTest.java
+++ b/model/src/test/java/lcsb/mapviewer/model/map/species/ProteinComparatorTest.java
@@ -6,6 +6,7 @@ import static org.junit.Assert.fail;
 
 import java.awt.geom.Point2D;
 
+import org.apache.log4j.Logger;
 import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
@@ -16,6 +17,7 @@ import lcsb.mapviewer.model.map.species.field.ModificationState;
 import lcsb.mapviewer.model.map.species.field.Residue;
 
 public class ProteinComparatorTest {
+  Logger logger=  Logger.getLogger(ProteinComparatorTest.class);
 
   ProteinComparator comparator = new ProteinComparator();
 
diff --git a/model/src/test/java/lcsb/mapviewer/model/map/species/RnaComparatorTest.java b/model/src/test/java/lcsb/mapviewer/model/map/species/RnaComparatorTest.java
index c28a417040c312d947f6068e7687df716a84cb6e..59ec62d18149ad8f01eefbaed3e55a38d7d679be 100644
--- a/model/src/test/java/lcsb/mapviewer/model/map/species/RnaComparatorTest.java
+++ b/model/src/test/java/lcsb/mapviewer/model/map/species/RnaComparatorTest.java
@@ -84,7 +84,6 @@ public class RnaComparatorTest {
     CodingRegion region1 = new CodingRegion();
     result.addRegion(region1);
     region1.setIdModificationResidue("a");
-    region1.setState(ModificationState.DONT_CARE);
     region1.setPosition(new Point2D.Double(0, 10));
     region1.setWidth(2.0);
     return result;
diff --git a/pathvisio/src/main/java/lcsb/mapviewer/wikipathway/XML/ModelContructor.java b/pathvisio/src/main/java/lcsb/mapviewer/wikipathway/XML/ModelContructor.java
index 90aed6106557938ce6114b8d4405f328d7987891..f2e88b01525577ae8be00329fa34db31aff8f33b 100644
--- a/pathvisio/src/main/java/lcsb/mapviewer/wikipathway/XML/ModelContructor.java
+++ b/pathvisio/src/main/java/lcsb/mapviewer/wikipathway/XML/ModelContructor.java
@@ -59,6 +59,7 @@ import lcsb.mapviewer.model.map.species.Rna;
 import lcsb.mapviewer.model.map.species.SimpleMolecule;
 import lcsb.mapviewer.model.map.species.Species;
 import lcsb.mapviewer.model.map.species.Unknown;
+import lcsb.mapviewer.model.map.species.field.AbstractSiteModification;
 import lcsb.mapviewer.model.map.species.field.ModificationResidue;
 import lcsb.mapviewer.model.map.species.field.ModificationSite;
 import lcsb.mapviewer.model.map.species.field.Residue;
@@ -385,7 +386,7 @@ public class ModelContructor {
       // TODO this might not work
       Element species = data.id2alias.get(state.getGraphRef());
       if (state.getType() != null) {
-        ModificationResidue mr = null;
+        AbstractSiteModification mr = null;
 
         if (species instanceof Protein) {
           mr = new Residue();
diff --git a/persist/src/main/resources/applicationContext-persist.xml b/persist/src/main/resources/applicationContext-persist.xml
index 933a3148a5adc6f19d8d3f5f178222fcd6389929..45c6df43f0f1b5ea2c0527f2d2c9ec1d19112c80 100644
--- a/persist/src/main/resources/applicationContext-persist.xml
+++ b/persist/src/main/resources/applicationContext-persist.xml
@@ -176,7 +176,10 @@
 				<value>lcsb.mapviewer.model.map.species.TruncatedProtein</value>
 				<value>lcsb.mapviewer.model.map.species.Unknown</value>
 
-				<value>lcsb.mapviewer.model.map.species.field.ModificationResidue</value>
+				<value>lcsb.mapviewer.model.map.species.field.CodingRegion</value>
+				<value>lcsb.mapviewer.model.map.species.field.ModificationSite</value>
+				<value>lcsb.mapviewer.model.map.species.field.ProteinBindingDomain</value>
+				<value>lcsb.mapviewer.model.map.species.field.Residue</value>
 				<value>lcsb.mapviewer.model.map.species.field.UniprotRecord</value>
 				<value>lcsb.mapviewer.model.map.species.field.Structure</value>
 				
diff --git a/persist/src/test/java/lcsb/mapviewer/persist/dao/map/layout/alias/AliasDaoTest2.java b/persist/src/test/java/lcsb/mapviewer/persist/dao/map/layout/alias/AliasDaoTest2.java
index 662ff26ee50e71691a8226c52a7c24fb596f2849..abab8f1ff18ba61d06e5297769f517dd076474b0 100644
--- a/persist/src/test/java/lcsb/mapviewer/persist/dao/map/layout/alias/AliasDaoTest2.java
+++ b/persist/src/test/java/lcsb/mapviewer/persist/dao/map/layout/alias/AliasDaoTest2.java
@@ -119,10 +119,11 @@ public class AliasDaoTest2 extends PersistTestFunctions {
       assertNotNull(sp2.getModificationResidues());
       assertEquals(1, sp2.getModificationResidues().size());
 
-      assertEquals(sp2.getModificationResidues().get(0).getPosition(), mr.getPosition());
-      assertEquals(sp2.getModificationResidues().get(0).getIdModificationResidue(), mr.getIdModificationResidue());
-      assertEquals(sp2.getModificationResidues().get(0).getName(), mr.getName());
-      assertEquals(sp2.getModificationResidues().get(0).getState(), mr.getState());
+      Residue copy = (Residue) sp2.getModificationResidues().get(0);
+      assertEquals(copy.getPosition(), mr.getPosition());
+      assertEquals(copy.getIdModificationResidue(), mr.getIdModificationResidue());
+      assertEquals(copy.getName(), mr.getName());
+      assertEquals(copy.getState(), mr.getState());
 
       elementDao.delete(sp2);
       sp2 = (Protein) elementDao.getById(protein.getId());
@@ -154,9 +155,10 @@ public class AliasDaoTest2 extends PersistTestFunctions {
       assertNotNull(sp2.getRegions());
       assertEquals(1, sp2.getRegions().size());
 
-      assertEquals(sp2.getRegions().get(0).getId(), mr.getId());
-      assertEquals(sp2.getRegions().get(0).getName(), mr.getName());
-      assertEquals(sp2.getRegions().get(0).getState(), mr.getState());
+      ModificationSite copy = (ModificationSite) sp2.getRegions().get(0);
+      assertEquals(copy.getId(), mr.getId());
+      assertEquals(copy.getName(), mr.getName());
+      assertEquals(copy.getState(), mr.getState());
 
       elementDao.delete(sp2);
       sp2 = (Rna) elementDao.getById(sp.getId());
diff --git a/rest-api/src/main/java/lcsb/mapviewer/api/projects/models/bioEntities/elements/ElementsRestImpl.java b/rest-api/src/main/java/lcsb/mapviewer/api/projects/models/bioEntities/elements/ElementsRestImpl.java
index 2e8302e6ac3cd3dee2f93854e0ec4cf6ab510b66..ab6009bf4c15319c410a2c619065deb7225f6422 100644
--- a/rest-api/src/main/java/lcsb/mapviewer/api/projects/models/bioEntities/elements/ElementsRestImpl.java
+++ b/rest-api/src/main/java/lcsb/mapviewer/api/projects/models/bioEntities/elements/ElementsRestImpl.java
@@ -20,7 +20,9 @@ import lcsb.mapviewer.model.map.species.Element;
 import lcsb.mapviewer.model.map.species.Protein;
 import lcsb.mapviewer.model.map.species.Rna;
 import lcsb.mapviewer.model.map.species.Species;
+import lcsb.mapviewer.model.map.species.field.AbstractSiteModification;
 import lcsb.mapviewer.model.map.species.field.ModificationResidue;
+import lcsb.mapviewer.model.map.species.field.ModificationState;
 import lcsb.mapviewer.model.map.species.field.Structure;
 import lcsb.mapviewer.model.map.species.field.UniprotRecord;
 import lcsb.mapviewer.services.SecurityException;
@@ -233,14 +235,14 @@ public class ElementsRestImpl extends BaseRestImpl {
   private List<Map<String, Object>> getModifications(List<? extends ModificationResidue> elements) {
     List<Map<String, Object>> result = new ArrayList<>();
     for (ModificationResidue region : elements) {
-      if (region.getState() != null) {
-        Map<String, Object> row = new TreeMap<>();
-        row.put("name", region.getName());
-        if (region.getState() != null) {
-          String state = region.getState().name();
-          row.put("state", state);
+      if (region instanceof AbstractSiteModification) {
+        ModificationState state = ((AbstractSiteModification) region).getState();
+        if (state != null) {
+          Map<String, Object> row = new TreeMap<>();
+          row.put("name", region.getName());
+          row.put("state", state.name());
+          result.add(row);
         }
-        result.add(row);
       }
     }
     return result;
diff --git a/model/src/main/java/lcsb/mapviewer/model/map/species/field/CodingRegion.java b/model/src/main/java/lcsb/mapviewer/model/map/species/field/CodingRegion.java
index 2f776c6a33845d5618bdac9f4a02921df3487f4f..1c15fdfaa11d2f4c9d32a0f9cf0d4a637f39e6c0 100644
--- a/model/src/main/java/lcsb/mapviewer/model/map/species/field/CodingRegion.java
+++ b/model/src/main/java/lcsb/mapviewer/model/map/species/field/CodingRegion.java
@@ -1,51 +1,45 @@
 package lcsb.mapviewer.model.map.species.field;
 
-import java.awt.geom.Point2D;
 import java.io.Serializable;
 
-import javax.persistence.Column;
 import javax.persistence.DiscriminatorValue;
 import javax.persistence.Entity;
 
 import org.apache.log4j.Logger;
 
-import lcsb.mapviewer.common.exception.InvalidArgumentException;
 import lcsb.mapviewer.common.exception.NotImplementedException;
 import lcsb.mapviewer.model.map.species.AntisenseRna;
+import lcsb.mapviewer.model.map.species.Gene;
+import lcsb.mapviewer.model.map.species.Rna;
+import lcsb.mapviewer.model.map.species.Species;
 
 /**
  * This structure contains information about antisense rna region (rna fragment
  * of interest) for a specific {@link AntisenseRna}.
  * 
  * @author Piotr Gawron
  * 
  */
 @Entity
 @DiscriminatorValue("MODIFICATION_SITE")
-public class CodingRegion extends ModificationResidue implements Serializable {
+public class CodingRegion extends AbstractRegionModification implements Serializable {
 
   /**
    * 
    */
   private static final long serialVersionUID = 1L;
 
-  /**
-   * Default size of the object (in graphical representation).
-   */
-  private static final double DEFAULT_SIZE = 10;
-
   /**
    * Default class logger.
    */
   @SuppressWarnings("unused")
   private static Logger logger = Logger.getLogger(CodingRegion.class);
 
-  /**
-   * Width of the region in the graphic representation.
-   */
-  @Column(name = "width")
-  private double width = DEFAULT_SIZE;
-
   /**
    * Default constructor.
    */
@@ -77,40 +71,4 @@ public class CodingRegion extends ModificationResidue implements Serializable {
     }
 
   }
-
-  /**
-   * Update data in this object from parameter (only if values in parameter object
-   * are valid).
-   * 
-   * @param mr
-   *          object from which we are updating data
-   */
-  public void update(CodingRegion mr) {
-    if (this.getIdModificationResidue() != null && !this.getIdModificationResidue().equals("")
-        && !this.getIdModificationResidue().equals(mr.getIdModificationResidue())) {
-      throw new InvalidArgumentException("Cannot update from mr with different id");
-    }
-    this.setWidth(mr.getWidth());
-    if (mr.getName() != null) {
-      this.setName(mr.getName());
-    }
-    if (mr.getPosition() != null) {
-      this.setPosition(mr.getPosition());
-    }
-
-  }
-
-  @Override
-  public String toString() {
-    String result = getIdModificationResidue() + "," + getName() + "," + getPosition() + "," + getWidth() + ",";
-    return result;
-  }
-
-  public double getWidth() {
-    return width;
-  }
-
-  public void setWidth(double width) {
-    this.width = width;
-  }
 }
@@ -1,51 +1,45 @@
 
 /**
- * This structure contains information about antisense rna region (rna fragment
- * of interest) for a specific {@link AntisenseRna}.
+ * This structure contains information about Coding Region for one of the
+ * following {@link Species}:
+ * <ul>
+ * <li>{@link Rna}</li>
+ * <li>{@link AntisenseRna}</li>
+ * <li>{@link Gene}</li>
+ * </ul>
  * 
  * @author Piotr Gawron
diff --git a/model/src/main/java/lcsb/mapviewer/model/map/species/field/ProteinBindingDomain.java b/model/src/main/java/lcsb/mapviewer/model/map/species/field/ProteinBindingDomain.java
index 6b6715e57c0929223930522358a9373226935a32..1803b107ecd1b3ea9fbaa0e9972f89496d4159a1 100644
--- a/model/src/main/java/lcsb/mapviewer/model/map/species/field/ProteinBindingDomain.java
+++ b/model/src/main/java/lcsb/mapviewer/model/map/species/field/ProteinBindingDomain.java
@@ -2,49 +2,42 @@ package lcsb.mapviewer.model.map.species.field;
 
 import java.io.Serializable;
 
-import javax.persistence.Column;
 import javax.persistence.DiscriminatorValue;
 import javax.persistence.Entity;
 
 import org.apache.log4j.Logger;
 
-import lcsb.mapviewer.common.exception.InvalidArgumentException;
 import lcsb.mapviewer.common.exception.NotImplementedException;
 import lcsb.mapviewer.model.map.species.AntisenseRna;
+import lcsb.mapviewer.model.map.species.Rna;
+import lcsb.mapviewer.model.map.species.Species;
 
 /**
  * This structure contains information about antisense rna region (rna fragment
  * of interest) for a specific {@link AntisenseRna}.
  * 
  * @author Piotr Gawron
  * 
  */
 @Entity
 @DiscriminatorValue("MODIFICATION_SITE")
-public class ProteinBindingDomain extends ModificationResidue implements Serializable {
+public class ProteinBindingDomain extends AbstractRegionModification implements Serializable {
 
   /**
    * 
    */
   private static final long serialVersionUID = 1L;
 
-  /**
-   * Default size of the object (in graphical representation).
-   */
-  private static final double DEFAULT_SIZE = 10;
-
   /**
    * Default class logger.
    */
   @SuppressWarnings("unused")
   private static Logger logger = Logger.getLogger(ProteinBindingDomain.class);
 
-  /**
-   * Width of the region in the graphic representation.
-   */
-  @Column(name = "width")
-  private double width = DEFAULT_SIZE;
-
   /**
    * Default constructor.
    */
@@ -60,7 +53,6 @@ public class ProteinBindingDomain extends ModificationResidue implements Seriali
    */
   public ProteinBindingDomain(ProteinBindingDomain original) {
     super(original);
-    this.setWidth(original.getWidth());
   }
 
   /**
@@ -76,40 +68,4 @@ public class ProteinBindingDomain extends ModificationResidue implements Seriali
     }
 
   }
-
-  /**
-   * Update data in this object from parameter (only if values in parameter object
-   * are valid).
-   * 
-   * @param mr
-   *          object from which we are updating data
-   */
-  public void update(ProteinBindingDomain mr) {
-    if (this.getIdModificationResidue() != null && !this.getIdModificationResidue().equals("")
-        && !this.getIdModificationResidue().equals(mr.getIdModificationResidue())) {
-      throw new InvalidArgumentException("Cannot update from mr with different id");
-    }
-    this.setWidth(mr.getWidth());
-    if (mr.getName() != null) {
-      this.setName(mr.getName());
-    }
-    if (mr.getPosition() != null) {
-      this.setPosition(mr.getPosition());
-    }
-
-  }
-
-  @Override
-  public String toString() {
-    String result = getIdModificationResidue() + "," + getName() + "," + getPosition() + "," + getWidth() + ",";
-    return result;
-  }
-
-  public double getWidth() {
-    return width;
-  }
-
-  public void setWidth(double width) {
-    this.width = width;
-  }
 }
@@ -2,49 +2,42 @@ package lcsb.mapviewer.model.map.species.field;
 
 /**
- * This structure contains information about antisense rna region (rna fragment
- * of interest) for a specific {@link AntisenseRna}.
+ * This structure contains information about Protein Binding Domain for one of
+ * the following {@link Species}:
+ * <ul>
+ * <li>{@link Rna}</li>
+ * <li>{@link AntisenseRna}</li>
+ * </ul>
  * 
  * @author Piotr Gawron